@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2008-2020 Oracle Corporation
+ * Copyright (C) 2008-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -133,11 +143,25 @@ static struct
     // Linuxes
     { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux,                VBOXOSTYPE_RedHat },
     { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux_64,             VBOXOSTYPE_RedHat_x64 },
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux,                VBOXOSTYPE_RedHat3 },
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux_64,             VBOXOSTYPE_RedHat3_x64 },
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux,                VBOXOSTYPE_RedHat4 },
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux_64,             VBOXOSTYPE_RedHat4_x64 },
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux,                VBOXOSTYPE_RedHat5 },
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux_64,             VBOXOSTYPE_RedHat5_x64 },
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux,                VBOXOSTYPE_RedHat6 },
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux_64,             VBOXOSTYPE_RedHat6_x64 },
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux_64,             VBOXOSTYPE_RedHat7_x64 }, // 64-bit only
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux_64,             VBOXOSTYPE_RedHat8_x64 }, // 64-bit only
+    { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux_64,             VBOXOSTYPE_RedHat9_x64 }, // 64-bit only
     { ovf::CIMOSType_CIMOS_SUSE,                                 VBOXOSTYPE_OpenSUSE },
-    { ovf::CIMOSType_CIMOS_SLES,                                 VBOXOSTYPE_OpenSUSE },
+    { ovf::CIMOSType_CIMOS_SLES,                                 VBOXOSTYPE_SUSE_LE },
     { ovf::CIMOSType_CIMOS_NovellLinuxDesktop,                   VBOXOSTYPE_OpenSUSE },
     { ovf::CIMOSType_CIMOS_SUSE_64,                              VBOXOSTYPE_OpenSUSE_x64 },
-    { ovf::CIMOSType_CIMOS_SLES_64,                              VBOXOSTYPE_OpenSUSE_x64 },
+    { ovf::CIMOSType_CIMOS_SLES_64,                              VBOXOSTYPE_SUSE_LE_x64 },
+    { ovf::CIMOSType_CIMOS_SUSE_64,                              VBOXOSTYPE_OpenSUSE_Leap_x64 }, // 64-bit only
+    { ovf::CIMOSType_CIMOS_SUSE,                                 VBOXOSTYPE_OpenSUSE_Tumbleweed },
+    { ovf::CIMOSType_CIMOS_SUSE_64,                              VBOXOSTYPE_OpenSUSE_Tumbleweed_x64 },
     { ovf::CIMOSType_CIMOS_LINUX,                                VBOXOSTYPE_Linux },
     { ovf::CIMOSType_CIMOS_LINUX,                                VBOXOSTYPE_Linux22 },
     { ovf::CIMOSType_CIMOS_SunJavaDesktopSystem,                 VBOXOSTYPE_Linux },
@@ -145,10 +169,75 @@ static struct
     { ovf::CIMOSType_CIMOS_TurboLinux_64,                        VBOXOSTYPE_Turbolinux_x64 },
     { ovf::CIMOSType_CIMOS_Mandriva,                             VBOXOSTYPE_Mandriva },
     { ovf::CIMOSType_CIMOS_Mandriva_64,                          VBOXOSTYPE_Mandriva_x64 },
+    { ovf::CIMOSType_CIMOS_Mandriva,                             VBOXOSTYPE_OpenMandriva_Lx },
+    { ovf::CIMOSType_CIMOS_Mandriva_64,                          VBOXOSTYPE_OpenMandriva_Lx_x64 },
+    { ovf::CIMOSType_CIMOS_Mandriva,                             VBOXOSTYPE_PCLinuxOS },
+    { ovf::CIMOSType_CIMOS_Mandriva_64,                          VBOXOSTYPE_PCLinuxOS_x64 },
+    { ovf::CIMOSType_CIMOS_Mandriva,                             VBOXOSTYPE_Mageia },
+    { ovf::CIMOSType_CIMOS_Mandriva_64,                          VBOXOSTYPE_Mageia_x64 },
     { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu },
     { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu10_LTS },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu10_LTS_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu10 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu10_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu11 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu11_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu12_LTS },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu12_LTS_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu12 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu12_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu13 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu13_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu14_LTS },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu14_LTS_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu14 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu14_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu15 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu15_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu16_LTS },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu16_LTS_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu16 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu16_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu17 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu17_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu18_LTS },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu18_LTS_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu18 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu18_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Ubuntu19 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu19_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu20_LTS_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu20_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu21_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu22_LTS_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu22_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Ubuntu23_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Lubuntu },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Lubuntu_x64 },
+    { ovf::CIMOSType_CIMOS_Ubuntu,                               VBOXOSTYPE_Xubuntu },
+    { ovf::CIMOSType_CIMOS_Ubuntu_64,                            VBOXOSTYPE_Xubuntu_x64 },
     { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian },
     { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian_x64 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian31 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian4 },
+    { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian4_x64 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian5 },
+    { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian5_x64 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian6 },
+    { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian6_x64 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian7 },
+    { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian7_x64 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian8 },
+    { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian8_x64 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian9 },
+    { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian9_x64 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian10 },
+    { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian10_x64 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian11 },
+    { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian11_x64 },
+    { ovf::CIMOSType_CIMOS_Debian,                               VBOXOSTYPE_Debian12 },
+    { ovf::CIMOSType_CIMOS_Debian_64,                            VBOXOSTYPE_Debian12_x64 },
     { ovf::CIMOSType_CIMOS_Linux_2_4_x,                          VBOXOSTYPE_Linux24 },
     { ovf::CIMOSType_CIMOS_Linux_2_4_x_64,                       VBOXOSTYPE_Linux24_x64 },
     { ovf::CIMOSType_CIMOS_Linux_2_6_x,                          VBOXOSTYPE_Linux26 },
@@ -177,6 +266,15 @@ static struct
     { ovf::CIMOSType_CIMOS_CentOS_64,                            VBOXOSTYPE_RedHat_x64 },
     { ovf::CIMOSType_CIMOS_OracleLinux,                          VBOXOSTYPE_Oracle },
     { ovf::CIMOSType_CIMOS_OracleLinux_64,                       VBOXOSTYPE_Oracle_x64 },
+    { ovf::CIMOSType_CIMOS_OracleLinux,                          VBOXOSTYPE_Oracle4 },
+    { ovf::CIMOSType_CIMOS_OracleLinux_64,                       VBOXOSTYPE_Oracle4_x64 },
+    { ovf::CIMOSType_CIMOS_OracleLinux,                          VBOXOSTYPE_Oracle5 },
+    { ovf::CIMOSType_CIMOS_OracleLinux_64,                       VBOXOSTYPE_Oracle5_x64 },
+    { ovf::CIMOSType_CIMOS_OracleLinux,                          VBOXOSTYPE_Oracle6 },
+    { ovf::CIMOSType_CIMOS_OracleLinux_64,                       VBOXOSTYPE_Oracle6_x64 },
+    { ovf::CIMOSType_CIMOS_OracleLinux_64,                       VBOXOSTYPE_Oracle7_x64 }, // 64-bit only
+    { ovf::CIMOSType_CIMOS_OracleLinux_64,                       VBOXOSTYPE_Oracle8_x64 }, // 64-bit only
+    { ovf::CIMOSType_CIMOS_OracleLinux_64,                       VBOXOSTYPE_Oracle9_x64 }, // 64-bit only
     { ovf::CIMOSType_CIMOS_eComStation,                          VBOXOSTYPE_ECS },
 
     { ovf::CIMOSType_CIMOS_WindowsServer2011,                    VBOXOSTYPE_Win2k8_x64 },       // no 1:1 match on the VBox side
@@ -355,6 +453,7 @@ Utf8Str convertNetworkAttachmentTypeToString(NetworkAttachmentType_T type)
         case NetworkAttachmentType_Bridged: strType = "Bridged"; break;
         case NetworkAttachmentType_Internal: strType = "Internal"; break;
         case NetworkAttachmentType_HostOnly: strType = "HostOnly"; break;
+        case NetworkAttachmentType_HostOnlyNetwork: strType = "HostOnlyNetwork"; break;
         case NetworkAttachmentType_Generic: strType = "Generic"; break;
         case NetworkAttachmentType_NATNetwork: strType = "NATNetwork"; break;
         case NetworkAttachmentType_Null: strType = "Null"; break;
@@ -451,7 +550,6 @@ HRESULT VirtualBox::createAppliance(ComPtr<IAppliance> &aAppliance)
  */
 HRESULT Appliance::init(VirtualBox *aVirtualBox)
 {
-    HRESULT rc = S_OK;
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
@@ -464,12 +562,12 @@ HRESULT Appliance::init(VirtualBox *aVirtualBox)
     m->m_pSecretKeyStore = new SecretKeyStore(false /* fRequireNonPageable*/);
     AssertReturn(m->m_pSecretKeyStore, E_FAIL);
 
-    rc = i_initBackendNames();
+    HRESULT hrc = i_initBackendNames();
 
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
 
-    return rc;
+    return hrc;
 }
 
 /**
@@ -608,7 +706,7 @@ HRESULT Appliance::createVFSExplorer(const com::Utf8Str &aURI, ComPtr<IVFSExplor
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     ComObjPtr<VFSExplorer> explorer;
-    HRESULT rc = S_OK;
+    HRESULT hrc;
     try
     {
         Utf8Str uri(aURI);
@@ -617,18 +715,18 @@ HRESULT Appliance::createVFSExplorer(const com::Utf8Str &aURI, ComPtr<IVFSExplor
         i_parseURI(aURI, li);
         /* Create the explorer object */
         explorer.createObject();
-        rc = explorer->init(li.storageType, li.strPath, li.strHostname, li.strUsername, li.strPassword, mVirtualBox);
+        hrc = explorer->init(li.storageType, li.strPath, li.strHostname, li.strUsername, li.strPassword, mVirtualBox);
     }
-    catch (HRESULT aRC)
+    catch (HRESULT hrcXcpt)
     {
-        rc = aRC;
+        hrc = hrcXcpt;
     }
 
-    if (SUCCEEDED(rc))
+    if (SUCCEEDED(hrc))
         /* Return explorer to the caller */
         explorer.queryInterfaceTo(aExplorer.asOutParam());
 
-    return rc;
+    return hrc;
 }
 
 
@@ -643,7 +741,7 @@ HRESULT Appliance::createVirtualSystemDescriptions(ULONG aRequested, ULONG *aCre
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    HRESULT rc = S_OK;
+    HRESULT hrc = S_OK;
     uint32_t lQuantity = aRequested;
     uint32_t i=0;
 
@@ -654,11 +752,11 @@ HRESULT Appliance::createVirtualSystemDescriptions(ULONG aRequested, ULONG *aCre
         for (; i<lQuantity; ++i)
         {
             ComObjPtr<VirtualSystemDescription> opVSD;
-            rc = opVSD.createObject();
-            if (SUCCEEDED(rc))
+            hrc = opVSD.createObject();
+            if (SUCCEEDED(hrc))
             {
-                rc = opVSD->init();
-                if (SUCCEEDED(rc))
+                hrc = opVSD->init();
+                if (SUCCEEDED(hrc))
                     m->virtualSystemDescriptions.push_back(opVSD);
                 else
                     break;
@@ -673,7 +771,7 @@ HRESULT Appliance::createVirtualSystemDescriptions(ULONG aRequested, ULONG *aCre
 
         *aCreated = i;
     }
-    catch (HRESULT aRC)
+    catch (HRESULT hrcXcpt)
     {
         for (; i>0; --i)
         {
@@ -682,10 +780,10 @@ HRESULT Appliance::createVirtualSystemDescriptions(ULONG aRequested, ULONG *aCre
             else
                 break;
         }
-        rc = aRC;
+        hrc = hrcXcpt;
     }
 
-    return rc;
+    return hrc;
 }
 
 /**
@@ -742,8 +840,8 @@ HRESULT Appliance::addPasswords(const std::vector<com::Utf8Str> &aIdentifiers,
     for (unsigned i = 0; i < aIdentifiers.size(); i++)
     {
         SecretKey *pKey = NULL;
-        int rc = m->m_pSecretKeyStore->retainSecretKey(aIdentifiers[i], &pKey);
-        if (rc != VERR_NOT_FOUND)
+        int vrc = m->m_pSecretKeyStore->retainSecretKey(aIdentifiers[i], &pKey);
+        if (vrc != VERR_NOT_FOUND)
         {
             AssertPtr(pKey);
             if (pKey)
@@ -757,13 +855,13 @@ HRESULT Appliance::addPasswords(const std::vector<com::Utf8Str> &aIdentifiers,
         size_t cbKey = aPasswords[i].length() + 1; /* Include terminator */
         const uint8_t *pbKey = (const uint8_t *)aPasswords[i].c_str();
 
-        int rc = m->m_pSecretKeyStore->addSecretKey(aIdentifiers[i], pbKey, cbKey);
-        if (RT_SUCCESS(rc))
+        int vrc = m->m_pSecretKeyStore->addSecretKey(aIdentifiers[i], pbKey, cbKey);
+        if (RT_SUCCESS(vrc))
             m->m_cPwProvided++;
-        else if (rc == VERR_NO_MEMORY)
+        else if (vrc == VERR_NO_MEMORY)
             hrc = setError(E_OUTOFMEMORY, tr("Failed to allocate enough secure memory for the key"));
         else
-            hrc = setError(E_FAIL, tr("Unknown error happened while adding a password (%Rrc)"), rc);
+            hrc = setErrorBoth(E_FAIL, vrc, tr("Unknown error happened while adding a password (%Rrc)"), vrc);
     }
 
     return hrc;
@@ -852,7 +950,7 @@ std::set<Utf8Str> Appliance::i_URIFromTypeOfVirtualDiskFormat(Utf8Str type)
  */
 HRESULT Appliance::i_findMediumFormatFromDiskImage(const ovf::DiskImage &di, ComObjPtr<MediumFormat>& mf)
 {
-    HRESULT rc = S_OK;
+    HRESULT hrc = S_OK;
 
     /* Get the system properties. */
     SystemProperties *pSysProps = mVirtualBox->i_getSystemProperties();
@@ -882,10 +980,9 @@ HRESULT Appliance::i_findMediumFormatFromDiskImage(const ovf::DiskImage &di, Com
             else
             {
                 mf.setNull();
-                rc = setError(E_FAIL,
-                              tr("Internal inconsistency looking up medium format for the disk image '%s'"),
-                              di.strHref.c_str());
-                return rc;
+                hrc = setError(E_FAIL, tr("Internal inconsistency looking up medium format for the disk image '%s'"),
+                               di.strHref.c_str());
+                return hrc;
             }
         }
         /* Figure out from extension which format the image of disk has. */
@@ -903,10 +1000,9 @@ HRESULT Appliance::i_findMediumFormatFromDiskImage(const ovf::DiskImage &di, Com
         mf = pSysProps->i_mediumFormat(strSrcFormat);
 
     if (mf.isNull())
-        rc = setError(E_FAIL, tr("Internal inconsistency looking up medium format for the disk image '%s'"),
-                      di.strHref.c_str());
+        hrc = setError(E_FAIL, tr("Internal inconsistency looking up medium format for the disk image '%s'"), di.strHref.c_str());
 
-    return rc;
+    return hrc;
 }
 
 /**
@@ -942,7 +1038,7 @@ RTVFSIOSTREAM Appliance::i_manifestSetupDigestCalculationForGivenIoStream(RTVFSI
     if (RT_SUCCESS(vrc))
         return hVfsIosPt;
 
-    setErrorVrc(vrc, "RTManifestEntryAddPassthruIoStream failed with rc=%Rrc", vrc);
+    setErrorVrc(vrc, tr("RTManifestEntryAddPassthruIoStream failed with vrc=%Rrc"), vrc);
     return NIL_RTVFSIOSTREAM;
 }
 
@@ -970,12 +1066,12 @@ bool Appliance::i_isApplianceIdle()
     return false;
 }
 
-HRESULT Appliance::i_searchUniqueVMName(Utf8Str& aName) const
+HRESULT Appliance::i_searchUniqueVMName(Utf8Str &aName) const
 {
-    IMachine *machine = NULL;
+    ComPtr<IMachine> ptrMachine;
     char *tmpName = RTStrDup(aName.c_str());
     int i = 1;
-    while (mVirtualBox->FindMachine(Bstr(tmpName).raw(), &machine) != VBOX_E_OBJECT_NOT_FOUND)
+    while (mVirtualBox->FindMachine(Bstr(tmpName).raw(), ptrMachine.asOutParam()) != VBOX_E_OBJECT_NOT_FOUND)
     {
         RTStrFree(tmpName);
         RTStrAPrintf(&tmpName, "%s %d", aName.c_str(), i);
@@ -987,19 +1083,19 @@ HRESULT Appliance::i_searchUniqueVMName(Utf8Str& aName) const
     return S_OK;
 }
 
-HRESULT Appliance::i_searchUniqueImageFilePath(const Utf8Str &aMachineFolder, DeviceType_T aDeviceType, Utf8Str &aName) const
+HRESULT Appliance::i_ensureUniqueImageFilePath(const Utf8Str &aMachineFolder, DeviceType_T aDeviceType, Utf8Str &aName) const
 {
     /*
      * Check if the file exists or if a medium with this path is registered already
      */
     Utf8Str strAbsName;
-    ssize_t offDashNum = -1;
-    ssize_t cchDashNum = 0;
+    size_t  offDashNum = ~(size_t)0;
+    size_t  cchDashNum = 0;
     for (unsigned i = 1;; i++)
     {
         /* Complete the path (could be relative to machine folder). */
-        int rc = RTPathAbsExCxx(strAbsName, aMachineFolder, aName);
-        AssertRCReturn(rc, Global::vboxStatusCodeToCOM(rc));  /** @todo stupid caller ignores this */
+        int vrc = RTPathAbsExCxx(strAbsName, aMachineFolder, aName);
+        AssertRCReturn(vrc, Global::vboxStatusCodeToCOM(vrc));  /** @todo stupid caller ignores this */
 
         /* Check that the file does not exist and that there is no media somehow matching the name. */
         if (!RTPathExists(strAbsName.c_str()))
@@ -1012,10 +1108,10 @@ HRESULT Appliance::i_searchUniqueImageFilePath(const Utf8Str &aMachineFolder, De
         }
 
         /* Insert '_%i' before the suffix and try again. */
-        if (offDashNum < 0)
+        if (offDashNum == ~(size_t)0)
         {
             const char *pszSuffix = RTPathSuffix(aName.c_str());
-            offDashNum = pszSuffix ? pszSuffix - aName.c_str() : aName.length();
+            offDashNum = pszSuffix ? (size_t)(pszSuffix - aName.c_str()) : aName.length();
         }
         char   szTmp[32];
         size_t cchTmp = RTStrPrintf(szTmp, sizeof(szTmp),  "_%u", i);
@@ -1032,14 +1128,14 @@ HRESULT Appliance::i_setUpProgress(ComObjPtr<Progress> &pProgress,
                                    const Utf8Str &strDescription,
                                    SetUpProgressMode mode)
 {
-    HRESULT rc;
+    HRESULT hrc;
 
     /* Create the progress object */
     try
     {
-        rc = pProgress.createObject();
-        if (FAILED(rc))
-            return rc;
+        hrc = pProgress.createObject();
+        if (FAILED(hrc))
+            return hrc;
     }
     catch (std::bad_alloc &)
     {
@@ -1240,53 +1336,53 @@ void Appliance::i_parseBucket(Utf8Str &aPath, Utf8Str &aBucket)
         case TaskOVF::Read:
             pAppliance->m->resetReadData();
             if (pTask->locInfo.storageType == VFSType_File)
-                pTask->rc = pAppliance->i_readFS(pTask);
+                pTask->hrc = pAppliance->i_readFS(pTask);
             else
-                pTask->rc = E_NOTIMPL;
+                pTask->hrc = E_NOTIMPL;
             break;
 
         case TaskOVF::Import:
             /** @todo allow overriding these? */
             if (!pAppliance->m->fSignatureValid && pAppliance->m->pbSignedDigest)
-                pTask->rc = pAppliance->setError(E_FAIL, tr("The manifest signature for '%s' is not valid"),
-                                                 pTask->locInfo.strPath.c_str());
+                pTask->hrc = pAppliance->setError(E_FAIL, tr("The manifest signature for '%s' is not valid"),
+                                                  pTask->locInfo.strPath.c_str());
             else if (!pAppliance->m->fCertificateValid && pAppliance->m->pbSignedDigest)
             {
                 if (pAppliance->m->strCertError.isNotEmpty())
-                    pTask->rc = pAppliance->setError(E_FAIL, tr("The certificate used to signed '%s' is not valid: %s"),
-                                                     pTask->locInfo.strPath.c_str(), pAppliance->m->strCertError.c_str());
+                    pTask->hrc = pAppliance->setError(E_FAIL, tr("The certificate used to signed '%s' is not valid: %s"),
+                                                      pTask->locInfo.strPath.c_str(), pAppliance->m->strCertError.c_str());
                 else
-                    pTask->rc = pAppliance->setError(E_FAIL, tr("The certificate used to signed '%s' is not valid"),
-                                                     pTask->locInfo.strPath.c_str());
+                    pTask->hrc = pAppliance->setError(E_FAIL, tr("The certificate used to signed '%s' is not valid"),
+                                                      pTask->locInfo.strPath.c_str());
             }
             // fusion does not consider this a show stopper (we've filed a warning during read).
             //else if (pAppliance->m->fCertificateMissingPath && pAppliance->m->pbSignedDigest)
-            //    pTask->rc = pAppliance->setError(E_FAIL, tr("The certificate used to signed '%s' is does not have a valid CA path"),
-            //                                     pTask->locInfo.strPath.c_str());
+            //    pTask->hrc = pAppliance->setError(E_FAIL, tr("The certificate used to signed '%s' is does not have a valid CA path"),
+            //                                      pTask->locInfo.strPath.c_str());
             else
             {
                 if (pTask->locInfo.storageType == VFSType_File)
-                    pTask->rc = pAppliance->i_importFS(pTask);
+                    pTask->hrc = pAppliance->i_importFS(pTask);
                 else
-                    pTask->rc = E_NOTIMPL;
+                    pTask->hrc = E_NOTIMPL;
             }
             break;
 
         case TaskOVF::Write:
             if (pTask->locInfo.storageType == VFSType_File)
-                pTask->rc = pAppliance->i_writeFS(pTask);
+                pTask->hrc = pAppliance->i_writeFS(pTask);
             else
-                pTask->rc = E_NOTIMPL;
+                pTask->hrc = E_NOTIMPL;
             break;
 
         default:
             AssertFailed();
-            pTask->rc = E_FAIL;
+            pTask->hrc = E_FAIL;
             break;
     }
 
     if (!pTask->pProgress.isNull())
-        pTask->pProgress->i_notifyComplete(pTask->rc);
+        pTask->pProgress->i_notifyComplete(pTask->hrc);
 
     LogFlowFuncLeave();
 }
@@ -1323,17 +1419,17 @@ void Appliance::i_exportOPCThreadTask(TaskOPC *pTask)
     switch (pTask->taskType)
     {
         case TaskOPC::Export:
-            pTask->rc = pAppliance->i_writeFSOPC(pTask);
+            pTask->hrc = pAppliance->i_writeFSOPC(pTask);
             break;
 
         default:
             AssertFailed();
-            pTask->rc = E_FAIL;
+            pTask->hrc = E_FAIL;
             break;
     }
 
     if (!pTask->pProgress.isNull())
-        pTask->pProgress->i_notifyComplete(pTask->rc);
+        pTask->pProgress->i_notifyComplete(pTask->hrc);
 
     LogFlowFuncLeave();
 }
@@ -1372,26 +1468,26 @@ void Appliance::i_importOrExportCloudThreadTask(TaskCloud *pTask)
     {
         case TaskCloud::Export:
             pAppliance->i_setApplianceState(ApplianceExporting);
-            pTask->rc = pAppliance->i_exportCloudImpl(pTask);
+            pTask->hrc = pAppliance->i_exportCloudImpl(pTask);
             break;
         case TaskCloud::Import:
             pAppliance->i_setApplianceState(ApplianceImporting);
-            pTask->rc = pAppliance->i_importCloudImpl(pTask);
+            pTask->hrc = pAppliance->i_importCloudImpl(pTask);
             break;
         case TaskCloud::ReadData:
             pAppliance->i_setApplianceState(ApplianceImporting);
-            pTask->rc = pAppliance->i_gettingCloudData(pTask);
+            pTask->hrc = pAppliance->i_gettingCloudData(pTask);
             break;
         default:
             AssertFailed();
-            pTask->rc = E_FAIL;
+            pTask->hrc = E_FAIL;
             break;
     }
 
     pAppliance->i_setApplianceState(ApplianceIdle);
 
     if (!pTask->pProgress.isNull())
-        pTask->pProgress->i_notifyComplete(pTask->rc);
+        pTask->pProgress->i_notifyComplete(pTask->hrc);
 
     LogFlowFuncLeave();
 }
@@ -1761,9 +1857,8 @@ void VirtualSystemDescription::i_removeByType(VirtualSystemDescriptionType_T aTy
  * the given reference ID. Useful when needing the controller for a particular
  * virtual disk.
  */
-const VirtualSystemDescriptionEntry* VirtualSystemDescription::i_findControllerFromID(uint32_t id)
+const VirtualSystemDescriptionEntry* VirtualSystemDescription::i_findControllerFromID(const Utf8Str &id)
 {
-    Utf8Str strRef = Utf8StrFmt("%RI32", id);
     vector<VirtualSystemDescriptionEntry>::const_iterator it;
     for (it = m->maDescriptions.begin();
          it != m->maDescriptions.end();
@@ -1777,7 +1872,7 @@ const VirtualSystemDescriptionEntry* VirtualSystemDescription::i_findControllerF
             case VirtualSystemDescriptionType_HardDiskControllerSCSI:
             case VirtualSystemDescriptionType_HardDiskControllerVirtioSCSI:
             case VirtualSystemDescriptionType_HardDiskControllerSAS:
-                if (d.strRef == strRef)
+                if (d.strRef == id)
                     return &d;
                 break;
             default: break; /* Shut up MSC. */
